-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
-- Script is the main part of the mission. You can't generally create a mission without some Lua
-- so get ready to some light to medium programming and add http://lua.org , Cortex Command Wiki
-- and forum search to the favourites in your browser.
--
-- This method\function is the place where the mission starts. Here you define and initialize variables, 
-- create scripted objects (brains, actors etc) and so on. If something goes wrong in this method, then
-- you kicked out to the campaign screen with error message in the console. Some times you need to restart
-- your mission twice, because CC will keep kicking you out complaining about already fixed errors.
-- Notice the name of the function, MissionTutorial_02 part is that Lua class name that you used in mission activities.ini
function MissionTutorial_02:StartActivity()
	-- Always print debugging information to the console but keep in mind that heavy prinintg
	-- causes heavy lag. Comment all the unnecessary prints when you release your mission to the public.
	-- If you notice some heavy unexpected lag, check the console. CC is probably flooding it with error messages.
	print("START! -- MissionTutorial_02:StartActivity()!");
	
	-- Areas is a region on the map. It may consist of one or many rectangles. You can check whether something is inside
	-- the area or not to trigger some events. You draw areas in area editor and don't have to edit ini files manually. 
	-- Here we obtain objects with data about areas defined in .\scenes\<scene name>.ini. 
	self.TriggerZone1 = SceneMan.Scene:GetArea("TriggerZone1");
	self.TriggerZone2 = SceneMan.Scene:GetArea("TriggerZone2");
	self.TriggerZone3 = SceneMan.Scene:GetArea("TriggerZone3");

	-- Get area with unique name to prevent CC engine from associating your activity with any other
	-- scenes except those for wich you designed your script. CC simply looks at areas defined in 
	-- scene and compares them to areas got in script with SceneMan.Scene:GetArea(). If everything
	-- matches then CC thinks that scene can be used with acitivty script and show scene on map for
	-- selected activity	
	self.Zone = SceneMan.Scene:GetArea("MissionTut_02_Area");
	
	-- Define enumeration with phases of mission. These are just friendly names for numbers 0,1,2,3 used to designate
	-- current mission phase. Nobody can stop you from using actual numbers (known as "magic numbers", because you can't
	-- determine what and how this number affects), but it is considered bad taste. Try using named variables and enumerations
	-- everywhere possible.
	self.FightPhase = { NOFIGHT = 0 , PHASEONE = 1 , PHASETWO = 2 , PHASETHREE = 3 };
    -- Set initial phase of mission.
	self.CurrentFightPhase = self.FightPhase.NOFIGHT;
	
	-- Define variables responsible for waiting phase. Note that we use variables instead of magic number here, so you have
	-- a single place (start function) where you can alter timings and gameplay specific variables
	self.SlotsToWait = 50; -- How many slots to wait
	self.SlotsWaited = 0; -- How many slots already waited
	self.SlotWaitTime = 250; -- Time in ms to wait for one slot. Player recieves one wait slot every 250 ms.
	
	-- Timers are essential in mission building. With timers you decide when you need to spawn enemies or dropship,
	-- when mission phase is over and anything else where you need time measuring. Timer is simple, you can reset it or
	-- find out how much time left scince last reset.
	self.WaitTimer = Timer(); -- Timer which triggers wait event
	
	-- Array which stores flags for players that lost their brain.
	self.braindead = {};
	-- Array which stores brains assigned to players. It is used for quick enumeration of 
	-- playable brains to trigger something or show mission marks.
	self.AssignedBrains = {};
	-- This is used to limit the amount of ingame movable objects (MO's). Currently CC goes mad if this number exceeds
	-- 255. To avoid unexpected behaviour you MUST always take care of the number of the movable ID's. We'll talk more
	-- about movable objects and limits later when we'll discuss spawning. Further reading - http://www.datarealms.com/wiki/index.php/Scenes
	self.MOIDLimit = 200;
    
	-- Here we store the amount of gold that player will recieve at start. Note that we not actualyy give player
	-- real gold here
	self.PlayerGold = 0; -- Player starting credits

   	-- Team variables are used just for convenience
   	-- Team 2 is always CPU
   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;

   	-- Set up players
    -- Here we create brains for players. Although current CC don't have predefined activities with multiplayer missions
	-- keep in mind that this behaviour may change, so you should treat gameplay as it was a multiplayer game so you'll have
	-- to fix less when CC will support standard multiplayer activities for missions
	for player = 0, self.PlayerCount - 1 do
        -- Check if we already have a brain assigned
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			-- Create brain here and alter all his neccessary properties
			local brain = CreateAHuman("Brain Robot" , "base.rte");
			-- Actor location on the scene. X=0 , Y=0 is the upper left corner of the map. Greater X is to the right.
			-- Greater Y is to the bottom.
			brain.Pos = Vector(150 + player * 30 , 550);
			-- Create weapon and give it to the actor.
			brain:AddInventoryItem(CreateHDFirearm("Auto Pistol" , "Coalition.rte"));
			-- Make brain just stay and do nothing. Further reading - http://www.datarealms.com/wiki/index.php/LuaDocs/Actor
			brain.AIMode = Actor.AIMODE_SENTRY;
			-- Add created brain to the array of assigned actors
			self.AssignedBrains[player] = brain;

            -- Let the spawn into the world, passing ownership
            MovableMan:AddActor(brain);
            -- Set the found brain to be the selected actor at start
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            -- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
			-- Select brain as current actor
			self:SwitchToActor(brain, player, self.PlayerTeam);
        end
    end
	
	-- Here we spawn another friendly clone
	local clone = CreateAHuman("Soldier Light" , "Coalition.rte");
	clone.Pos = Vector(100 , 550);
	clone.Team = self.PlayerTeam;
	clone.AIMode = Actor.AIMODE_SENTRY;
	MovableMan:AddActor(clone);
	
    -- Set the team's funds. Here we actually give a player some gold.
    self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function MissionTutorial_02:PauseActivity(pause)
    print("PAUSE! -- MissionTutorial_02:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function MissionTutorial_02:EndActivity()
    print("END! -- MissionTutorial_02:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
-- This is the main loop of the mission. This function is called many times at a second and
-- processes all mission events. You should not write heavy code here, just call some specific functions
-- with actual mission code. Nobody can prevent you from writing all mission code here, but at some moment
-- you'll find that you missed some end somewhere, and you have to check 1000 linies of code to find out where,
-- because Lua gives you wring line numbers in error message
function MissionTutorial_02:UpdateActivity()
   	-- Clear all objective markers, they get re-added each frame
   	self:ClearObjectivePoints();

	-- Depending on the current mission phase we call different function that processes mission gameplay. You can write
	-- all mission processing in a single function with a bunch of deep random if's but as soon as your mission
	-- will become more and more complicated you will emit more and more f-words into th air. For every phase you may need
	-- a gameplay processing function (PhaseOne) and a function that switches gameplay to that phase (AdvanceToOne).
	-- Gameplay processing function checks triggers, shows messages, spawns enemies and such, while advance-function usually alters 
	-- mission variables (spawn times etc), enables\disables mission features (dropships, land spawns, particle cannons, 
	-- reinforcements), resets neccessary timers, adds funds etc. These are not mandatory, just a way to save some brain cells.
	if self.CurrentFightPhase == self.FightPhase.NOFIGHT then
		self:PhaseNoFight();
	elseif self.CurrentFightPhase == self.FightPhase.PHASEONE then
		self:PhaseOne();
	elseif self.CurrentFightPhase == self.FightPhase.PHASETWO then
		self:PhaseTwo();
	elseif self.CurrentFightPhase == self.FightPhase.PHASETHREE then
		self:PhaseThree();
	end
	
	-- Check for defeat conditions
	self:DoBrainSelection();
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
-- Here we check if we stil have a brain to cobtrol our troops. If not - game over
function MissionTutorial_02:DoBrainSelection()
	-- Check if mission is still running to prevent mission failure
	-- if you waste brain after victory or lose it's object after evacuation
	if not (self.ActivityState == Activity.OVER) then
		-- Iterate through all human players
		for player = 0, self.PlayerCount - 1 do
			-- The current player's team
			local team = self:GetTeamOfPlayer(player);
	
			-- Make sure the game is not already ending
			-- Check if any player's brain is dead
			local brain = self:GetPlayerBrain(player);
			-- Notice the MovableMan:IsActor(brain) function call. This is eesntial when you reference
			-- ingame objects especially actors from lua. It checks if onjects still exists. If you reference 
			-- properties of a non-existent actor (actor can die at any moment) the game will simply crash. 
			-- No fancy console errors at all, just a silent crash.
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				-- Reset player brain
				self:SetPlayerBrain(nil, player);
				-- Try to find a new unasigned brain this player can use instead, or if his old brain entered a craft
				local newBrain = MovableMan:GetUnassignedBrain(team);
				-- Found new brain actor, assign it and keep on truckin'
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					-- All brains of this team are dead. Show fail reason
					FrameMan:SetScreenText("Commander died!", player, 100, -1, false);
					-- Set the flag that this player is brainless
					self.braindead[player] = true;
					
					-- Now see if all brains of self player's team are dead, and if so, end the game
					-- Show Fail\Win banner to participating teams. 
					self.WinnerTeam = self:OtherTeam(team);
					-- Game over
					ActivityMan:EndActivity();
				end
			else
				-- Update the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Phase "No fight"
-----------------------------------------------------------------------------------------
-- This is the initial mission phase. "Historicaly" it is called "no fight".
function MissionTutorial_02:PhaseNoFight()
	-- Show a fixed objective mark with down arrow
	self:AddObjectivePoint("PUT BRAIN HERE", Vector(408 , 490) , self.PlayerTeam, GameActivity.ARROWDOWN);
	
	-- Enumerate all player brains and determine if one of them is inside the area. If it is, advance to next phase
	for player = 0, self.PlayerCount - 1 do
		if self.TriggerZone1:IsInside(self.AssignedBrains[player].Pos) then
			self:AdvanceToOne();
		end
	end
end
-----------------------------------------------------------------------------------------
-- "No fight" -> "Phase one"
-----------------------------------------------------------------------------------------
-- This is a typical phase switching function. It shows messages and switches to next phase. It's easier to have this one
-- instead of a bunch of code in phase processing mission, so you always know where you need to alter some difficulty for
-- mission phase.
function MissionTutorial_02:AdvanceToOne()
	-- Switch to next phase
	self.CurrentFightPhase = self.FightPhase.PHASEONE;
	
	-- Show non flashing message in the center of the screen for all players for 5000 ms. This is one of the places
	-- where you need to take multiple players into account. Consider making a stand alone function with this code for
	-- printing, but remember that sometimes you need to switch between flashing and non-flashing text messages to 
	-- attract player to the text
	for player = 0, self.PlayerCount - 1 do
		-- Clear previous message and reset message timer
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		-- Put message text
		FrameMan:SetScreenText("NON FLASHING INFO MESSAGE", player, 0, 5000, true);
	end	
end
-----------------------------------------------------------------------------------------
-- Phase "One"
-----------------------------------------------------------------------------------------
function MissionTutorial_02:PhaseOne()
	-- Loop through all actors to find an actor from our team. Be cautious with this loop and use it wiseley as it
	-- may add signifiant lag if used many times especially when you have many actors.
	for actor in MovableMan.Actors do
		-- Check if an actor is from our team
		if actor.Team == self.PlayerTeam then
			-- Show objective mark above an actor. Notice the AboveHUDPos property which frees you from playing
			-- with Vector arithmetics to place an arrow above an actor.
			self:AddObjectivePoint("PUT ANY ACTOR THERE", actor.AboveHUDPos , self.PlayerTeam, GameActivity.ARROWRIGHT);
		
			-- Check if an actor from our team is inside an area, if it is advance to next phase
			if self.TriggerZone2:IsInside(actor.Pos) then
				self:AdvanceToTwo();
			end
		end
	end

	-- Show fixed objective mark for current phase. Don't be lazy to add these for major objectives so player chopping
	-- zombies with minigun always know for what. Keep in mind though, that many arrow can easily confuse player
	self:AddObjectivePoint("PUT ANY ACTOR HERE", Vector(660, 522) , self.PlayerTeam, GameActivity.ARROWDOWN);
end
-----------------------------------------------------------------------------------------
-- "No fight" -> "Phase two"
-----------------------------------------------------------------------------------------
function MissionTutorial_02:AdvanceToTwo()
	self.CurrentFightPhase = self.FightPhase.PHASETWO;
	
	for player = 0, self.PlayerCount - 1 do
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		-- Put flashing message text, that flashes avery 500 ms
		FrameMan:SetScreenText("FLASHING INFO MESSAGE", player, 500, 5000, true);
	end	
	
	-- We will use a timer in the next phase, so we need to reset it here to avoid instant triggering
	-- when next phase will be active
	self.WaitTimer:Reset();
end
-----------------------------------------------------------------------------------------
-- Phase "Two"
-----------------------------------------------------------------------------------------
function MissionTutorial_02:PhaseTwo()
	-- This phase is a little bit complex as we trigger events with timer and area.
	-- First we need to determine if we have any of our actor inside the area.
	local actorinside;
	-- Assume no actor is inside
	actorinside = false;
	
	for actor in MovableMan.Actors do
		if self.TriggerZone3:IsInside(actor.Pos) then
			-- Set the flag that we found an actor
			actorinside = true;
			-- Stop looping actors because we already found what we need
			break;
		end
	end -- for

	if actorinside then
		self:AddObjectivePoint("FLASHING TEXT ABOVE", Vector(912, 480) , self.PlayerTeam, GameActivity.ARROWUP);

		-- Check if enuogh time left scince last reset to trigger wait chunk increment. We need to check if an actor is
		-- inside the zone to stop increasing wait chunk if an actor leaves and continue from where we started when
		-- he returns
		if self.WaitTimer:IsPastSimMS(self.SlotWaitTime) then
			-- Check if an actor waited for enough time
			if self.SlotsWaited < self.SlotsToWait then
				-- Increase amount of completed waitng
				self.SlotsWaited = self.SlotsWaited + 1;
				
				-- Show player's progress on top of the screen. If player needs to do something for a long period of time
				-- you need to inform him that everything is fine and maybe show his progress. If you re lazy enough not
				-- to do so, unstable player may panic, decide that mission is broken or something similar.
				for player = 0, self.PlayerCount - 1 do
					self:ResetMessageTimer(player);
					FrameMan:ClearScreenText(player);
					-- Notice that string.formal is used here to build a string which contain text and numbers from variable
					FrameMan:SetScreenText(string.format("%d WAIT SLOTS LEFT" , self.SlotsToWait - self.SlotsWaited) , player, 333, -1, false);
				end					
				
				-- Reset timer because the event was triggered and we need to wait for another
				self.WaitTimer:Reset();
			else
				-- Check if our activity already finished. This will save us from endless triggering of victory once all
				-- objectives complete. You may not notice this bug, but this is erroneous behavoiur and it prevents
				-- victory music from playing
				if not (self.ActivityState == Activity.OVER) then
					-- Set the winner team
					self.WinnerTeam = self.PlayerTeam;
					-- End current activity. It disables the AI but the UpdateActivity is still running and everything works
					ActivityMan:EndActivity();
					-- Tell the script to stop running for brains as the game has ended.
					for player = 0, self.PlayerCount - 1 do
						self.braindead[player] = true;
					end -- for
				end -- if
			end -- else
		end -- if  -- If you have very deep hierarchy of if's and for's you can add these comments at the end of each block
	else           -- to make your laife little bit easier
		-- We have two objective marks for this mission, one is shown when player is inside the area, second when he is outside
		self:AddObjectivePoint("STAY HERE FOR A WHILE", Vector(912, 520) , self.PlayerTeam, GameActivity.ARROWDOWN);
		-- Reset wait timer because player is not inside the area so it does not make instant trigger hit if player 
		-- returned inside the zone after some walking. This way player loses current slot if he leaves wait area
		self.WaitTimer:Reset();
	end -- else
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































